@@ -28,7 +28,7 @@
 #' @details The linear \code{linearWeight} and \code{logisticWeight} weight functions 
 #' can be passed to \code{\link[dtwSat]{twdtwApply}} through the argument \code{weight.fun}. 
 #' This will add a time-weight to the dynamic time warping analysis. The time weight 
-#' creates a global constraint useful to analyse time series with phenological cycles
+#' creates a global constraint useful to analyze time series with phenological cycles
 #' of vegetation that are usually bound to seasons. In previous studies by 
 #' \insertCite{Maus:2016;textual}{dtwSat} the logistic weight had better results than the 
 #' linear for land cover classification. See \insertCite{Maus:2016;textual}{dtwSat} and 
@@ -45,6 +45,6 @@
 #' 
 #' @export
 logisticWeight = function(alpha, beta){
-  function(psi) 1 / (1 + exp(alpha * (psi - beta )))
+  function(phi, psi) phi + 1 / (1 + exp(alpha * (psi - beta )))
 }
 
